/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.eventbus;

import static com.google.common.base.Throwables.propagate;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class MethodEventSubscription implements EventSubscription {
    private final Class<? extends Event> eventClass;
    private final Object object;
    private final Method method;

    public MethodEventSubscription(Object object, Method method) {
        eventClass = eventClass(method);
        checkMethod(method);
        this.object = object;
        this.method = method;
        method.setAccessible(true);
    }

    private static void checkMethod(Method m) {
        checkReturnType(m);
    }

    @SuppressWarnings("unchecked")
    private static Class<? extends Event> eventClass(Method m) {
        Class<?>[] p = m.getParameterTypes();
        if (p.length != 1 && !Event.class.isAssignableFrom(p.getClass()))
            throw new IllegalArgumentException("Method " + m.getName() + " must have exactly one argument of a subtype of " + Event.class.getName());
        return (Class<? extends Event>)p[0];
    }

    private static void checkReturnType(Method m) {
        Class<?> c = m.getReturnType();
        if (!isVoid(c))
            throw new IllegalArgumentException("Method " +m.getName() + " has an unexpected return type");
    }

    private static boolean isVoid(Class<?> c) {
        return c == Void.class || c.getName().equals("void");
    }


    @Override public void handleEvent(Event event) {
        try {
            method.invoke(object, event);
        } catch (IllegalArgumentException x) { throw new Error("Method "+method+" rejected argument '"+event+"'", x);
        } catch (IllegalAccessException x) { throw new Error("Method "+method+" is inaccessible: " +event, x);
        } catch (InvocationTargetException x) {
//            if (x.getCause() instanceof Error)
//                throw (Error)x.getCause();
            throw propagate(x.getCause() == null? x : x.getCause());
        }
    }

    public Class<? extends Event> getEventClass() {
        return eventClass;
    }

    public Object getObject() {
        return object;
    }

    public Method getMethod() {
        return method;
    }

    @Override public String toString() {
        return method.getDeclaringClass().getSimpleName() +"."+ method.getName() +"("+ eventClass.getSimpleName() +")";
    }
}
